@@ -52,7 +52,7 @@
 #include "zlib.h"
 #include "libusb.h"
 
-int FastBoot::Transport(string cmd, void *p, size_t size, vector<uint8_t> *input)
+int FastBoot::Transport(const string &cmd, void *p, size_t size, vector<uint8_t> *input)
 {
 	if (m_pTrans->write((void*)cmd.data(), cmd.size()))
 		return -1;
@@ -66,6 +66,8 @@ int FastBoot::Transport(string cmd, void *p, size_t size, vector<uint8_t> *input
 		memset(buff, 0, 65);
 		if (m_pTrans->read(buff, 64, &actual))
 			return -1;
+	
+		buff[actual] = '\0';
 
 		if (strncmp(buff, "DATA",4) == 0)
 		{
@@ -176,7 +178,7 @@ int FBCmd::parser(char *p)
 	{
 		string err = "Unknown command: ";
 		err += s;
-		set_last_err_string(s);
+		set_last_err_string(err);
 		return -1;
 	}
 
@@ -302,7 +304,7 @@ int FBCopy::parser(char *p)
 	{
 		string err = "Unknown command: ";
 		err += s;
-		set_last_err_string(s);
+		set_last_err_string(err);
 		return -1;
 	}
 
@@ -717,8 +719,15 @@ int FBFlashCmd::run(CmdCtx *ctx)
 			size_t block_size = 4096;
 			if (getvar.parser((char*)"FB: getvar logical-block-size"))
 				return -1;
-			if (!getvar.run(ctx))
-				block_size = str_to_uint32(getvar.m_val);
+
+			if (!getvar.run(ctx)) {
+				bool conversion_succeeded = false;
+				size_t tmp_block_size = str_to_uint32(getvar.m_val, &conversion_succeeded);
+
+				if (conversion_succeeded) {
+					block_size = tmp_block_size;
+				}
+			}
 
 			if (block_size == 0) {
 				set_last_err_string("Device report block_size is 0");
@@ -727,7 +736,7 @@ int FBFlashCmd::run(CmdCtx *ctx)
 
 			m_bmap = bmap_t{SIZE_MAX, block_size};
 		// load mappings from the file
-		} else if (!load_bmap(m_bmap_filename, m_bmap)) {
+		} else if (load_bmap(m_bmap_filename, m_bmap)) {
 			set_last_err_string("Failed to load BMAP");
 			return -1;
 		}
@@ -810,7 +819,9 @@ int FBFlashCmd::run(CmdCtx *ctx)
 		pb = pin->request_data(0, sizeof(sparse_header));
 		if (!pb)
 			return -1;
-		sparse_header * pfile = (sparse_header *)pb->data();
+
+		sparse_header header = *(sparse_header *)pb->data();
+		sparse_header * pfile = &header;
 
 		if (!SparseFile::is_validate_sparse_file(pb->data(), sizeof(sparse_header)))
 		{
@@ -1058,7 +1069,7 @@ int FBFlashCmd::flash_ffu(FastBoot *fb, shared_ptr<FileBuffer> pin)
 	return 0;
 }
 
-FBLoop::FBLoop(char* p): CmdBase(p)
+FBLoop::FBLoop(const char* p): CmdBase(p)
 {
 	insert_param_info("-f", &m_filename, Param::Type::e_string_filename);
 	insert_param_info("-format", &m_uboot_cmd, Param::Type::e_string);
@@ -1069,23 +1080,32 @@ FBLoop::FBLoop(char* p): CmdBase(p)
 	insert_param_info("-nostop", &m_nostop, Param::Type::e_bool);
 }
 
-string FBLoop::build_cmd(string& cmd, size_t off, size_t sz)
+string FBLoop::build_cmd(const string& cmd, size_t off, size_t sz)
 {
 	string ucmd="UCmd: ";
 	ucmd += cmd;
+	size_t pos;
 
 	string_ex ex;
 
-	size_t pos= ucmd.find("@off");
-	if (pos != string::npos) {
-		ex.format("0x%llx", off);
-		ucmd = ucmd.replace(pos, 4, ex);
+	while(1) {
+		pos= ucmd.find("@off");
+		if (pos != string::npos) {
+			ex.format("0x%llx", off);
+			ucmd = ucmd.replace(pos, 4, ex);
+		} else {
+			break;
+		}
 	}
 
-	pos = ucmd.find("@size");
-	if (pos != string::npos) {
-		ex.format("0x%llx", sz);
-		ucmd = ucmd.replace(pos, 5, ex);
+	while (1) {
+		pos = ucmd.find("@size");
+		if (pos != string::npos) {
+			ex.format("0x%llx", sz);
+			ucmd = ucmd.replace(pos, 5, ex);
+		} else {
+			break;
+		}
 	}
 
 	return ucmd;
