@@ -63,11 +63,12 @@ struct ROM_INFO
 	const char * m_name;
 	uint32_t    free_addr;
 	uint32_t	flags;
+	int serial_idx;
 };
 
 const ROM_INFO * search_rom_info(const std::string &s);
 const ROM_INFO * search_rom_info(const ConfigItem *item);
 
-size_t GetContainerActualSize(std::shared_ptr<DataBuffer> p, size_t offset, bool bROMAPI=false);
+size_t GetContainerActualSize(std::shared_ptr<DataBuffer> p, size_t offset, bool bROMAPI=false, bool skipspl=false);
 size_t GetFlashHeaderSize(std::shared_ptr<DataBuffer> p, size_t offset = 0);
 
